/*
   331CA Crecana Constantin.Cosmin
*/

#include <stdio.h>
#include <stdlib.h>
#include <math.h>
#include <avr/io.h>
#include <avr/pgmspace.h>
#include <avr/interrupt.h>
#include <util/delay.h>
#include "mpu6050/mpu6050.h"

#include "DataTypes.h"
#include "DefaultFonts.h"
#include "UTFT.h"
#include "sound/pff.h"

#define WIDTH 5

#define LIMIT 0.25

#define BORDER_MIN_X 14
#define BORDER_MIN_Y 54
#define BORDER_MAX_X 310
#define BORDER_MAX_Y 230

#define GO_LEFT 1
#define GO_RIGHT 2
#define GO_UP 3
#define GO_DOWN 4

#define START_X 160
#define START_Y 90

#define TILT 0.7
#define NO_FOOD 15
#define DIM 960
#define ROWS 240

using namespace std;

extern const unsigned char SmallFont[];

    double score;
    double best_score = 0;

typedef struct {
    int x;
    int y;
} pixel;

pixel circles[NO_FOOD];

pixel my_coordinates, my_prev_coordinates;
pixel oponent, oponent_prev_coordinates;
double my_size = 10;
pixel snake[100];
pixel food;
char dir = GO_RIGHT, prev_dir = GO_RIGHT;
int size = 3, maxscore = 0;

bool eat = false;

bool lose = false;

UTFT myGLCD;


void getRandPosition(int *x, int *y)
{
    *x = (BORDER_MIN_X + rand() % (BORDER_MAX_X - BORDER_MIN_X + 1)) / 5 * 5;
    *y = (BORDER_MIN_Y + rand() % (BORDER_MAX_Y - BORDER_MIN_Y + 1)) / 5 * 5;
}  
double getDistance(pixel p1, pixel p2)
{
    double dx = p1.x - p2.x;
    double dy = p1.y - p2.y;
    return sqrt(dx * dx + dy * dy);
}

bool checkConsume(pixel p, int me)
{
    bool found = false;
    for(int i = 0; i < NO_FOOD; ++i) {
        if (getDistance(my_coordinates, circles[i]) < my_size) {
            /* Delete old circle */
                 found = true;
                 score += 1;
                 if (score > best_score) {
                    best_score = score;
                 }
            int x, y;
            myGLCD.setColor(255, 255, 255);
            myGLCD.fillCircle(circles[i].x, circles[i].y, 6);
            getRandPosition(&x, &y);
            pixel new_pixel;
            new_pixel.x = x;
            new_pixel.y = y;
            circles[i] = new_pixel;
            myGLCD.setColor(rand() % 256, rand() % 256 , rand() % 256);
            myGLCD.fillCircle(x, y, 6);
            if (me)
                my_size += 0.5;
        }
    }
    return found;
}

bool is_inside_borders(pixel p)
{
	if (p.x + WIDTH >= BORDER_MAX_X ||
		p.x - WIDTH <= BORDER_MIN_X ||
		p.y + WIDTH >= BORDER_MAX_Y ||
		p.y - WIDTH <= BORDER_MIN_Y) {

		return false;
	}

	return true;
}

int checkDefeat()
{
    double d = getDistance(oponent, my_coordinates);
    if (d < my_size + 10) {
        return 1;
    }
    return 0;
}
void move_oponent()
{
    double dx, dy, length;
    checkConsume(oponent, 0);
    dx = oponent.x - my_coordinates.x;
    dy = oponent.y - my_coordinates.y;

    length = getDistance(my_coordinates, oponent);
    dx /= length;
    dy /= length;

    oponent.x = oponent.x - 3 * dx;
    oponent.y = oponent.y - 3 * dy;

    if(!is_inside_borders(oponent)) {
        oponent.x = oponent_prev_coordinates.x;
        oponent.y = oponent_prev_coordinates.y;
    }
    myGLCD.setColor(255, 255, 255);
    myGLCD.fillCircle(oponent_prev_coordinates.x, oponent_prev_coordinates.y, 10);
    myGLCD.setColor(255, 0, 0);
    myGLCD.fillCircle(oponent.x, oponent.y, 10);
    oponent_prev_coordinates.x = oponent.x;
    oponent_prev_coordinates.y = oponent.y;
}

void move_snake()
{

	switch (dir) {
        myGLCD.setColor(0, 0, 0);
		case GO_RIGHT: {
				my_coordinates.x += WIDTH;
			break;
		}
		case GO_LEFT: {
			my_coordinates.x -= WIDTH;
            break;
		}
		case GO_UP: {
			my_coordinates.y -= WIDTH;
            break;
		}
		case GO_DOWN: {
			my_coordinates.y += WIDTH;
			break;
		}
	}

	if (!is_inside_borders(my_coordinates)) {
		my_coordinates.x = my_prev_coordinates.x;
        my_coordinates.y = my_prev_coordinates.y;
	}
    checkConsume(my_coordinates, 1);
}



void detect_direction(double axg, double ayg)
{
    if(axg >  LIMIT)
        dir = GO_DOWN;
    else if(axg < -LIMIT)
        dir = GO_UP;
    if(ayg >  LIMIT)
        dir = GO_LEFT;
    else if(ayg < -LIMIT)
        dir = GO_RIGHT;
}

void init_playground()
{
    byte a = 255;
    myGLCD.clrScr();
    myGLCD.fillScr(255, 255, 255);
    myGLCD.setColor(255, 0 , 0);
    myGLCD.setColor(255, 0, 0);

    myGLCD.print("AgarIo PM!", 40, 10);

    int i;
    for (i = 0; i < NO_FOOD; ++i)
    {
        int x, y;
        getRandPosition(&x, &y);
        pixel new_pixel;
        new_pixel.x = x;
        new_pixel.y = y;
        circles[i] = new_pixel;
        myGLCD.setColor(rand() % 256, rand() % 256 , rand() % 256);
        myGLCD.fillCircle(x, y, 6);
    }
  	lose = false;
  	eat = false;
  	dir = GO_RIGHT; prev_dir = GO_RIGHT;
  	size = 3;
}


FATFS fs;

void reset() {
    size = 15;
}

int main( void )
{
    double axg = 0;
    double ayg = 0;
    double azg = 0;
    double gxds = 0;
    double gyds = 0;
    double gzds = 0;
    double prev_ayg = 0;
    double my_prev_size = my_size;
    char itmp[10];

	// Initialize Acceletometer and LCD
	mpu6050_init();
    myGLCD.InitLCD();
  	myGLCD.setFont(SmallFont);

  	// mount sd card file sistem
    for(;;)
	{
		if(pf_mount(&fs) != FR_OK) // mount filesystem
		{
			_delay_ms(1000); // wait a while and retry
			continue;
		} else break;
	}

  	//get_music(music_folder);

  	char buff[DIM];

    // initialize buffer
    for(int i = 0; i< DIM; i++)
        buff[i] = '\0';

  	FRESULT ret;
  	int p;

    // read image and print it
	if((ret = pf_open("logo.ppm")) == FR_OK)
	{
       // myGLCD.print("am deschis", 100, 50);

        WORD nrread;

        // read header
        pf_read(buff, 15, &nrread);

        //myGLCD.print(buff, 100,60);

        for(int i = 0; i < ROWS; i ++) {
            pf_read(buff, DIM, &nrread);

            p = 0;

            for(int j = 0; j < DIM; j += 3) {
                p ++;
                myGLCD.setColor(buff[j], buff[j+1], buff[j+2]);

                myGLCD.drawPixel(p, i);
            }
        }

	}

    myGLCD.print("Tilt to start", 200, 20);

    while(1) {
        mpu6050_getConvData(&axg, &ayg, &azg, &gxds, &gyds, &gzds);

        if(ayg - prev_ayg > TILT)
            break;

        prev_ayg = ayg;

        _delay_ms(100);
    }

    srand(0);
    best_score = 0;
start:
    score = 0;
    init_playground();
    my_size = 10;
    my_prev_size = 10;
    getRandPosition(&my_coordinates.x, &my_coordinates.y);
    my_prev_coordinates.x = my_coordinates.x;
    my_prev_coordinates.y = my_coordinates.y;
    myGLCD.setColor(0, 255, 0);

    myGLCD.fillCircle(my_coordinates.x, my_coordinates.y, my_size);

    getRandPosition(&oponent.x, &oponent.y);
    oponent_prev_coordinates.x = oponent_prev_coordinates.y;
    my_prev_coordinates.y = my_coordinates.y;
    myGLCD.setColor(255, 0, 0);

    myGLCD.fillCircle(oponent.x, oponent.y, 10);

    while(1) {
        myGLCD.setColor(255, 0, 0);
        myGLCD.print("Score: ", 150, 10);
        myGLCD.printNumI(score, 250, 10);
        myGLCD.print("Best score: ", 150, 20);
        myGLCD.printNumI(best_score, 250, 20);
        /* Delete prev position */
        myGLCD.setColor(255, 255, 255);
        myGLCD.fillCircle(my_prev_coordinates.x, my_prev_coordinates.y, my_prev_size);

        myGLCD.setColor(0, 255, 0);
        myGLCD.fillCircle(my_coordinates.x, my_coordinates.y, my_size);
        mpu6050_getConvData(&axg, &ayg, &azg, &gxds, &gyds, &gzds);
        detect_direction(axg, ayg);
        my_prev_coordinates.x = my_coordinates.x;
        my_prev_coordinates.y = my_coordinates.y;
        my_prev_size = my_size;
        move_snake();
        if (my_size > 75)
            break;
        move_oponent();
        if (checkDefeat()) {
            myGLCD.print("You lose", my_coordinates.x-20, my_coordinates.y - 10);
            break;
        }
        _delay_ms(150);
    }
    myGLCD.print("Tilt to start again", my_coordinates.x-20, my_coordinates.y);
    while(1) {
        mpu6050_getConvData(&axg, &ayg, &azg, &gxds, &gyds, &gzds);

        if(ayg - prev_ayg > TILT)
            break;

        prev_ayg = ayg;

        _delay_ms(100);
    }

    goto start;
 }
